@@ -20,48 +20,46 @@ import Hledger.Data.Account (nullacct)
 import Hledger.Data.AccountName
 import Hledger.Data.Journal
 import Hledger.Data.Posting
-import Hledger.Data.Matching
+import Hledger.Query
 
 
 instance Show Ledger where
     show l = printf "Ledger with %d transactions, %d accounts\n%s"
-             (length (jtxns $ journal l) +
-              length (jmodifiertxns $ journal l) +
-              length (jperiodictxns $ journal l))
-             (length $ accountnames l)
-             (showtree $ accountnametree l)
+             (length (jtxns $ ledgerJournal l) +
+              length (jmodifiertxns $ ledgerJournal l) +
+              length (jperiodictxns $ ledgerJournal l))
+             (length $ ledgerAccountNames l)
+             (showtree $ ledgerAccountNameTree l)
 
 nullledger :: Ledger
 nullledger = Ledger{
-      journal = nulljournal,
-      accountnametree = nullaccountnametree,
-      accountmap = fromList []
+      ledgerJournal = nulljournal,
+      ledgerAccountNameTree = nullaccountnametree,
+      ledgerAccountMap = fromList []
     }
 
 -- | Filter a journal's transactions as specified, and then process them
 -- to derive a ledger containing all balances, the chart of accounts,
 -- canonicalised commodities etc.
-journalToLedger :: FilterSpec -> Journal -> Ledger
-journalToLedger fs j = nullledger{journal=j',accountnametree=t,accountmap=m}
-    where j' = filterJournalPostings fs{depth=Nothing} j
-          (t, m) = journalAccountInfo j'
-
--- | Filter a journal's transactions as specified, and then process them
--- to derive a ledger containing all balances, the chart of accounts,
--- canonicalised commodities etc.
--- Like journalToLedger but uses the new matchers.
-journalToLedger2 :: Matcher -> Journal -> Ledger
-journalToLedger2 m j = nullledger{journal=j',accountnametree=t,accountmap=amap}
-    where j' = filterJournalPostings2 m j
+journalToLedger :: Query -> Journal -> Ledger
+journalToLedger q j = nullledger{ledgerJournal=j',ledgerAccountNameTree=t,ledgerAccountMap=amap}
+    where j' = filterJournalPostings q j
           (t, amap) = journalAccountInfo j'
 
+tests_journalToLedger = [
+ "journalToLedger" ~: do
+  assertEqual "" (0) (length $ ledgerPostings $ journalToLedger Any nulljournal)
+  assertEqual "" (11) (length $ ledgerPostings $ journalToLedger Any samplejournal)
+  assertEqual "" (6) (length $ ledgerPostings $ journalToLedger (Depth 2) samplejournal)
+ ]
+
 -- | List a ledger's account names.
 ledgerAccountNames :: Ledger -> [AccountName]
-ledgerAccountNames = drop 1 . flatten . accountnametree
+ledgerAccountNames = drop 1 . flatten . ledgerAccountNameTree
 
 -- | Get the named account from a ledger.
 ledgerAccount :: Ledger -> AccountName -> Account
-ledgerAccount l a = findWithDefault nullacct a $ accountmap l
+ledgerAccount l a = findWithDefault nullacct a $ ledgerAccountMap l
 
 -- | List a ledger's accounts, in tree order
 ledgerAccounts :: Ledger -> [Account]
@@ -77,20 +75,20 @@ ledgerLeafAccounts = leaves . ledgerAccountTree 9999
 
 -- | Accounts in ledger whose name matches the pattern, in tree order.
 ledgerAccountsMatching :: [String] -> Ledger -> [Account]
-ledgerAccountsMatching pats = filter (matchpats pats . aname) . accounts
+ledgerAccountsMatching pats = filter (matchpats pats . aname) . ledgerAccounts
 
 -- | List a ledger account's immediate subaccounts
 ledgerSubAccounts :: Ledger -> Account -> [Account]
 ledgerSubAccounts l Account{aname=a} = 
-    map (ledgerAccount l) $ filter (`isSubAccountNameOf` a) $ accountnames l
+    map (ledgerAccount l) $ filter (`isSubAccountNameOf` a) $ ledgerAccountNames l
 
 -- | List a ledger's postings, in the order parsed.
 ledgerPostings :: Ledger -> [Posting]
-ledgerPostings = journalPostings . journal
+ledgerPostings = journalPostings . ledgerJournal
 
 -- | Get a ledger's tree of accounts to the specified depth.
 ledgerAccountTree :: Int -> Ledger -> Tree Account
-ledgerAccountTree depth l = treemap (ledgerAccount l) $ treeprune depth $ accountnametree l
+ledgerAccountTree depth l = treemap (ledgerAccount l) $ treeprune depth $ ledgerAccountNameTree l
 
 -- | Get a ledger's tree of accounts rooted at the specified account.
 ledgerAccountTreeAt :: Ledger -> Account -> Maybe (Tree Account)
@@ -101,47 +99,10 @@ ledgerAccountTreeAt l acct = subtreeat acct $ ledgerAccountTree 9999 l
 ledgerDateSpan :: Ledger -> DateSpan
 ledgerDateSpan = postingsDateSpan . ledgerPostings
 
--- | Convenience aliases.
-accountnames :: Ledger -> [AccountName]
-accountnames = ledgerAccountNames
+-- | All commodities used in this ledger, as a map keyed by symbol.
+ledgerCommodities :: Ledger -> Map String Commodity
+ledgerCommodities = journalCanonicalCommodities . ledgerJournal
 
-account :: Ledger -> AccountName -> Account
-account = ledgerAccount
-
-accounts :: Ledger -> [Account]
-accounts = ledgerAccounts
-
-topaccounts :: Ledger -> [Account]
-topaccounts = ledgerTopAccounts
-
-accountsmatching :: [String] -> Ledger -> [Account]
-accountsmatching = ledgerAccountsMatching
-
-subaccounts :: Ledger -> Account -> [Account]
-subaccounts = ledgerSubAccounts
-
-postings :: Ledger -> [Posting]
-postings = ledgerPostings
-
-commodities :: Ledger -> Map String Commodity
-commodities = journalCanonicalCommodities . journal
-
-accounttree :: Int -> Ledger -> Tree Account
-accounttree = ledgerAccountTree
-
-accounttreeat :: Ledger -> Account -> Maybe (Tree Account)
-accounttreeat = ledgerAccountTreeAt
-
--- datespan :: Ledger -> DateSpan
--- datespan = ledgerDateSpan
-
-rawdatespan :: Ledger -> DateSpan
-rawdatespan = journalDateSpan . journal
-
-ledgeramounts :: Ledger -> [MixedAmount]
-ledgeramounts = journalAmounts . journal
-
-tests_Hledger_Data_Ledger = TestList
- [
- ]
+tests_Hledger_Data_Ledger = TestList $
+    tests_journalToLedger
 
