@@ -1,14 +1,47 @@
 {-|
 
-A 'Posting' represents a 'MixedAmount' being added to or subtracted from a
-single 'Account'.  Each 'Transaction' contains two or more postings which
-should add up to 0. Postings also reference their parent transaction, so
-we can get a date or description for a posting (from the transaction).
-Strictly speaking, \"entry\" is probably a better name for these.
+A 'Posting' represents a change (by some 'MixedAmount') of the balance in
+some 'Account'.  Each 'Transaction' contains two or more postings which
+should add up to 0. Postings reference their parent transaction, so we can
+look up the date or description there.
 
 -}
 
-module Hledger.Data.Posting
+module Hledger.Data.Posting (
+  -- * Posting
+  nullposting,
+  -- * operations
+  postingCleared,
+  isReal,
+  isVirtual,
+  isBalancedVirtual,
+  isEmptyPosting,
+  hasAmount,
+  postingAllTags,
+  transactionAllTags,
+  -- * date operations
+  postingDate,
+  isPostingInDateSpan,
+  postingsDateSpan,
+  -- * account name operations
+  accountNamesFromPostings,
+  accountNamePostingType,
+  accountNameWithoutPostingType,
+  accountNameWithPostingType,
+  joinAccountNames,
+  concatAccountNames,
+  accountNameApplyAliases,
+  -- * arithmetic
+  sumPostings,
+  -- * rendering
+  showPosting,
+  showPostingForRegister,
+  -- * misc.
+  postingTagsAsLines,
+  tagsAsLines,
+  showComment,
+  tests_Hledger_Data_Posting
+)
 where
 import Data.List
 import Data.Ord
@@ -30,8 +63,8 @@ nullposting :: Posting
 nullposting = Posting False "" nullmixedamt "" RegularPosting [] Nothing
 
 showPosting :: Posting -> String
-showPosting (Posting{paccount=a,pamount=amt,pcomment=com,ptype=t}) =
-    concatTopPadded [showaccountname a ++ " ", showamount amt, comment]
+showPosting p@Posting{paccount=a,pamount=amt,ptype=t} =
+    unlines $ [concatTopPadded [showaccountname a ++ " ", showamount amt, showComment (pcomment p)]] ++ postingTagsAsLines p
     where
       ledger3ishlayout = False
       acctnamewidth = if ledger3ishlayout then 25 else 22
@@ -41,7 +74,17 @@ showPosting (Posting{paccount=a,pamount=amt,pcomment=com,ptype=t}) =
                           VirtualPosting -> (\s -> "("++s++")", acctnamewidth-2)
                           _ -> (id,acctnamewidth)
       showamount = padleft 12 . showMixedAmount
-      comment = if null com then "" else "  ; " ++ com
+
+
+postingTagsAsLines :: Posting -> [String]
+postingTagsAsLines = tagsAsLines . ptags
+
+tagsAsLines :: [(String, String)] -> [String]
+tagsAsLines mds = map (\(k,v) -> "    ; " ++ k++": "++v) mds
+
+showComment :: String -> String
+showComment s = if null s then "" else "  ; " ++ s
+
 -- XXX refactor
 showPostingForRegister :: Posting -> String
 showPostingForRegister (Posting{paccount=a,pamount=amt,ptype=t}) =
@@ -66,7 +109,7 @@ isBalancedVirtual :: Posting -> Bool
 isBalancedVirtual p = ptype p == BalancedVirtualPosting
 
 hasAmount :: Posting -> Bool
-hasAmount = (/= missingamt) . pamount
+hasAmount = (/= missingmixedamt) . pamount
 
 accountNamesFromPostings :: [Posting] -> [AccountName]
 accountNamesFromPostings = nub . map paccount
@@ -86,6 +129,14 @@ postingCleared p = if pstatus p
                     then True
                     else maybe False tstatus $ ptransaction p
 
+-- | Tags for this posting including any inherited from its parent transaction.
+postingAllTags :: Posting -> [Tag]
+postingAllTags p = ptags p ++ maybe [] transactionAllTags (ptransaction p)
+
+-- | Tags for this transaction including any inherited from above, when that is implemented.
+transactionAllTags :: Transaction -> [Tag]
+transactionAllTags t = ttags t
+
 -- | Does this posting fall within the given date span ?
 isPostingInDateSpan :: DateSpan -> Posting -> Bool
 isPostingInDateSpan s = spanContainsDate s . postingDate
@@ -93,8 +144,8 @@ isPostingInDateSpan s = spanContainsDate s . postingDate
 isEmptyPosting :: Posting -> Bool
 isEmptyPosting = isZeroMixedAmount . pamount
 
--- | Get the minimal date span which contains all the postings, or
--- DateSpan Nothing Nothing if there are none.
+-- | Get the minimal date span which contains all the postings, or the
+-- null date span if there are none.
 postingsDateSpan :: [Posting] -> DateSpan
 postingsDateSpan [] = DateSpan Nothing Nothing
 postingsDateSpan ps = DateSpan (Just $ postingDate $ head ps') (Just $ addDays 1 $ postingDate $ last ps')
