@@ -1,12 +1,39 @@
 {-|
 
-A 'Transaction' consists of two or more related 'Posting's which balance
-to zero, representing a movement of some commodity(ies) between accounts,
-plus a date and optional metadata like description and cleared status.
+A 'Transaction' represents a movement of some commodity(ies) between two
+or more accounts. It consists of multiple account 'Posting's which balance
+to zero, a date, and optional extras like description, cleared status, and
+tags.
 
 -}
 
-module Hledger.Data.Transaction
+module Hledger.Data.Transaction (
+  -- * Transaction
+  nulltransaction,
+  txnTieKnot,
+  -- settxn,
+  -- * operations
+  showAccountName,
+  hasRealPostings,
+  realPostings,
+  virtualPostings,
+  balancedVirtualPostings,
+  transactionsPostings,
+  isTransactionBalanced,
+  -- nonzerobalanceerror,
+  -- * date operations
+  transactionActualDate,
+  transactionEffectiveDate,
+  journalTransactionWithDate,
+  -- * arithmetic
+  transactionPostingBalances,
+  balanceTransaction,
+  -- * rendering
+  showTransaction,
+  showTransactionUnelided,
+  -- * misc.
+  tests_Hledger_Data_Transaction
+)
 where
 import Data.List
 import Data.Maybe
@@ -38,7 +65,7 @@ nulltransaction = Transaction {
                     tcode="", 
                     tdescription="", 
                     tcomment="",
-                    tmetadata=[],
+                    ttags=[],
                     tpostings=[],
                     tpreceding_comment_lines=""
                   }
@@ -65,35 +92,113 @@ showTransaction = showTransaction' True
 showTransactionUnelided :: Transaction -> String
 showTransactionUnelided = showTransaction' False
 
+tests_showTransactionUnelided = [
+   "showTransactionUnelided" ~: do
+    let t `gives` s = assertEqual "" s (showTransactionUnelided t)
+    nulltransaction `gives` "0000/01/01\n\n"
+    nulltransaction{
+      tdate=parsedate "2012/05/14",
+      teffectivedate=Just $ parsedate "2012/05/15",
+      tstatus=False,
+      tcode="code",
+      tdescription="desc",
+      tcomment="tcomment1\ntcomment2\n",
+      ttags=[("ttag1","val1")],
+      tpostings=[
+        nullposting{
+          pstatus=True,
+          paccount="a",
+          pamount=Mixed [dollars 1, hours 2],
+          pcomment="pcomment1\npcomment2\n",
+          ptype=RegularPosting,
+          ptags=[("ptag1","val1"),("ptag2","val2")]
+          }
+       ]
+      }
+      `gives` unlines [
+      "2012/05/14=2012/05/15 (code) desc  ; tcomment1",
+      "    ; tcomment2",
+      "    ; ttag1: val1",
+      "                $1.00",
+      "    * a          2.0h  ; pcomment1",
+      "    ; pcomment2",
+      "    ; ptag1: val1",
+      "    ; ptag2: val2",
+      ""
+      ]
+ ]
+
+-- XXX overlaps showPosting
 showTransaction' :: Bool -> Transaction -> String
 showTransaction' elide t =
-    unlines $ [description] ++ showpostings (tpostings t) ++ [""]
+    unlines $ [descriptionline]
+              ++ commentlines
+              ++ (tagsAsLines $ ttags t)
+              ++ (postingsAsLines elide t (tpostings t))
+              ++ [""]
     where
-      description = concat [date, status, code, desc, comment]
+      descriptionline = rstrip $ concat [date, status, code, desc, firstcomment]
       date = showdate (tdate t) ++ maybe "" showedate (teffectivedate t)
       showdate = printf "%-10s" . showDate
       showedate = printf "=%s" . showdate
       status = if tstatus t then " *" else ""
       code = if length (tcode t) > 0 then printf " (%s)" $ tcode t else ""
       desc = if null d then "" else " " ++ d where d = tdescription t
-      comment = if null c then "" else "  ; " ++ c where c = tcomment t
-      showpostings ps
-          | elide && length ps > 1 && isTransactionBalanced Nothing t -- imprecise balanced check
-              = map showposting (init ps) ++ [showpostingnoamt (last ps)]
-          | otherwise = map showposting ps
-          where
-            showpostingnoamt p = rstrip $ showacct p ++ "              " ++ showcomment (pcomment p)
-            showposting p = concatTopPadded [showacct p
-                                            ,"  "
-                                            ,showamt (pamount p)
-                                            ,showcomment (pcomment p)
-                                            ]
-            showacct p = "    " ++ showstatus p ++ printf (printf "%%-%ds" w) (showAccountName Nothing (ptype p) (paccount p))
-                where w = maximum $ map (length . paccount) ps
-                      showstatus p = if pstatus p then "* " else ""
-            showamt =
-                padleft 12 . showMixedAmount
-            showcomment s = if null s then "" else "  ; "++s
+      (firstcomment, commentlines) = commentLines $ tcomment t
+
+-- Render a transaction or posting's comment as indented & prefixed comment lines.
+commentLines :: String -> (String, [String])
+commentLines s
+    | null s = ("", [])
+    | otherwise = ("  ; " ++ first, map (indent . ("; "++)) rest)
+    where (first:rest) = lines s
+
+postingsAsLines :: Bool -> Transaction -> [Posting] -> [String]
+postingsAsLines elide t ps
+    | elide && length ps > 1 && isTransactionBalanced Nothing t -- imprecise balanced check
+       = (concatMap (postingAsLines False ps) $ init ps) ++ postingAsLines True ps (last ps)
+    | otherwise = concatMap (postingAsLines False ps) ps
+
+postingAsLines :: Bool -> [Posting] -> Posting -> [String]
+postingAsLines elideamount ps p =
+    postinglines
+    ++ commentlines
+    ++ tagsAsLines (ptags p)
+  where
+    postinglines = map rstrip $ lines $ concatTopPadded [showacct p, "  ", amount, firstcomment]
+    amount = if elideamount then "" else showamt (pamount p)
+    (firstcomment, commentlines) = commentLines $ pcomment p
+    showacct p =
+      indent $ showstatus p ++ printf (printf "%%-%ds" w) (showAccountName Nothing (ptype p) (paccount p))
+        where
+          showstatus p = if pstatus p then "* " else ""
+          w = maximum $ map (length . paccount) ps
+    showamt =
+        padleft 12 . showMixedAmount
+
+tests_postingAsLines = [
+   "postingAsLines" ~: do
+    let p `gives` ls = assertEqual "" ls (postingAsLines False [p] p)
+    nullposting `gives` ["                 0"]
+    nullposting{
+      pstatus=True,
+      paccount="a",
+      pamount=Mixed [dollars 1, hours 2],
+      pcomment="pcomment1\npcomment2\n",
+      ptype=RegularPosting,
+      ptags=[("ptag1","val1"),("ptag2","val2")]
+      }
+     `gives` [
+      "                $1.00",
+      "    * a          2.0h  ; pcomment1",
+      "    ; pcomment2",
+      "    ; ptag1: val1",
+      "    ; ptag2: val2"
+      ]      
+ ]
+
+indent :: String -> String
+indent = ("    "++)
 
 -- | Show an account name, clipped to the given width if any, and
 -- appropriately bracketed/parenthesised for the given posting type.
@@ -232,9 +337,9 @@ nonzerobalanceerror t = printf "could not balance this transaction (%s%s%s)" rms
     where
       (rsum, _, bvsum) = transactionPostingBalances t
       rmsg | isReallyZeroMixedAmountCost rsum = ""
-           | otherwise = "real postings are off by " ++ show (costOfMixedAmount rsum)
+           | otherwise = "real postings are off by " ++ showMixedAmount (costOfMixedAmount rsum)
       bvmsg | isReallyZeroMixedAmountCost bvsum = ""
-            | otherwise = "balanced virtual postings are off by " ++ show (costOfMixedAmount bvsum)
+            | otherwise = "balanced virtual postings are off by " ++ showMixedAmount (costOfMixedAmount bvsum)
       sep = if not (null rmsg) && not (null bvmsg) then "; " else ""
 
 transactionActualDate :: Transaction -> Day
@@ -258,7 +363,10 @@ txnTieKnot t@Transaction{tpostings=ps} = t{tpostings=map (settxn t) ps}
 settxn :: Transaction -> Posting -> Posting
 settxn t p = p{ptransaction=Just t}
 
-tests_Hledger_Data_Transaction = TestList [
+tests_Hledger_Data_Transaction = TestList $ concat [
+  tests_postingAsLines,
+  tests_showTransactionUnelided,
+  [
   "showTransaction" ~: do
      assertEqual "show a balanced transaction, eliding last amount"
        (unlines
@@ -318,12 +426,12 @@ tests_Hledger_Data_Transaction = TestList [
      assertEqual "show a transaction with one posting and a missing amount"
        (unlines
         ["2007/01/28 coopportunity"
-        ,"    expenses:food:groceries              "
+        ,"    expenses:food:groceries"
         ,""
         ])
        (showTransaction
         (txnTieKnot $ Transaction (parsedate "2007/01/28") Nothing False "" "coopportunity" "" []
-         [Posting False "expenses:food:groceries" missingamt "" RegularPosting [] Nothing
+         [Posting False "expenses:food:groceries" missingmixedamt "" RegularPosting [] Nothing
          ] ""))
 
   ,"showTransaction" ~: do
@@ -331,13 +439,13 @@ tests_Hledger_Data_Transaction = TestList [
        (unlines
         ["2010/01/01 x"
         ,"    a        1 @ $2"
-        ,"    b              "
+        ,"    b"
         ,""
         ])
        (showTransaction
         (txnTieKnot $ Transaction (parsedate "2010/01/01") Nothing False "" "x" "" []
          [Posting False "a" (Mixed [Amount unknown 1 (Just $ UnitPrice $ Mixed [Amount dollar{precision=0} 2 Nothing])]) "" RegularPosting [] Nothing
-         ,Posting False "b" missingamt "" RegularPosting [] Nothing
+         ,Posting False "b" missingmixedamt "" RegularPosting [] Nothing
          ] ""))
 
   ,"balanceTransaction" ~: do
@@ -350,12 +458,12 @@ tests_Hledger_Data_Transaction = TestList [
      assertBool "detect unbalanced entry, multiple missing amounts"
                     (isLeft $ balanceTransaction Nothing
                            (Transaction (parsedate "2007/01/28") Nothing False "" "test" "" []
-                            [Posting False "a" missingamt "" RegularPosting [] Nothing,
-                             Posting False "b" missingamt "" RegularPosting [] Nothing
+                            [Posting False "a" missingmixedamt "" RegularPosting [] Nothing,
+                             Posting False "b" missingmixedamt "" RegularPosting [] Nothing
                             ] ""))
      let e = balanceTransaction Nothing (Transaction (parsedate "2007/01/28") Nothing False "" "" "" []
                            [Posting False "a" (Mixed [dollars 1]) "" RegularPosting [] Nothing,
-                            Posting False "b" missingamt "" RegularPosting [] Nothing
+                            Posting False "b" missingmixedamt "" RegularPosting [] Nothing
                            ] "")
      assertBool "balanceTransaction allows one missing amount" (isRight e)
      assertEqual "balancing amount is inferred"
@@ -417,4 +525,4 @@ tests_Hledger_Data_Transaction = TestList [
              ] ""
      assertBool "balanced virtual postings need to balance among themselves (2)" (isTransactionBalanced Nothing t)
 
-  ]
+  ]]
