@@ -69,6 +69,8 @@ data Commodity = Commodity {
       separatorpositions :: [Int]  -- ^ positions of separators, counting leftward from decimal point
     } deriving (Eq,Ord,Show,Read)
 
+type Quantity = Double
+
 -- | An amount's price in another commodity may be written as \@ unit
 -- price or \@\@ total price.  Note although a MixedAmount is used, it
 -- should be in a single commodity, also the amount should be positive;
@@ -78,7 +80,7 @@ data Price = UnitPrice MixedAmount | TotalPrice MixedAmount
 
 data Amount = Amount {
       commodity :: Commodity,
-      quantity :: Double,
+      quantity :: Quantity,
       price :: Maybe Price  -- ^ the price for this amount at posting time
     } deriving (Eq,Ord)
 
@@ -87,13 +89,15 @@ newtype MixedAmount = Mixed [Amount] deriving (Eq,Ord)
 data PostingType = RegularPosting | VirtualPosting | BalancedVirtualPosting
                    deriving (Eq,Show)
 
+type Tag = (String, String)
+
 data Posting = Posting {
       pstatus :: Bool,
       paccount :: AccountName,
       pamount :: MixedAmount,
-      pcomment :: String,
+      pcomment :: String, -- ^ this posting's non-tag comment lines, as a single non-indented string
       ptype :: PostingType,
-      pmetadata :: [(String,String)],
+      ptags :: [Tag],
       ptransaction :: Maybe Transaction  -- ^ this posting's parent transaction (co-recursive types).
                                         -- Tying this knot gets tedious, Maybe makes it easier/optional.
     }
@@ -109,8 +113,8 @@ data Transaction = Transaction {
       tstatus :: Bool,  -- XXX tcleared ?
       tcode :: String,
       tdescription :: String,
-      tcomment :: String,
-      tmetadata :: [(String,String)],
+      tcomment :: String, -- ^ this transaction's non-tag comment lines, as a single non-indented string
+      ttags :: [Tag],
       tpostings :: [Posting],            -- ^ this transaction's postings (co-recursive types).
       tpreceding_comment_lines :: String
     } deriving (Eq)
@@ -163,9 +167,9 @@ data Journal = Journal {
       final_comment_lines :: String,        -- ^ any trailing comments from the journal file
       jContext :: JournalContext,           -- ^ the context (parse state) at the end of parsing
       files :: [(FilePath, String)],        -- ^ the file path and raw text of the main and
-                                           -- any included journal files. The main file is
-                                           -- first followed by any included files in the
-                                           -- order encountered.
+                                            -- any included journal files. The main file is
+                                            -- first followed by any included files in the
+                                            -- order encountered (XXX reversed, cf journalAddFile).
       filereadtime :: ClockTime             -- ^ when this journal was last read from its file(s)
     } deriving (Eq, Typeable)
 
@@ -173,33 +177,86 @@ data Journal = Journal {
 -- raise an error.
 type JournalUpdate = ErrorT String IO (Journal -> Journal)
 
+-- | The id of a data format understood by hledger, eg @journal@ or @csv@.
+type Format = String
+
 -- | A hledger journal reader is a triple of format name, format-detecting
 -- predicate, and a parser to Journal.
-data Reader = Reader {rFormat   :: String
-                     ,rDetector :: FilePath -> String -> Bool
-                     ,rParser   :: FilePath -> String -> ErrorT String IO Journal
-                     }
+data Reader = Reader {
+     -- name of the format this reader handles
+     rFormat   :: Format
+     -- quickly check if this reader can probably handle the given file path and file content
+    ,rDetector :: FilePath -> String -> Bool
+     -- parse the given string, using the given parse rules file if any, returning a journal or error aware of the given file path
+    ,rParser   :: Maybe FilePath -> FilePath -> String -> ErrorT String IO Journal
+    }
+
+instance Show Reader where show r = "Reader for "++rFormat r
+
+-- data format parse/conversion rules
+
+-- currently the only parse (conversion) rules are those for the CSV format
+type ParseRules = CsvRules
+
+-- XXX copied from Convert.hs
+{- |
+A set of data definitions and account-matching patterns sufficient to
+convert a particular CSV data file into meaningful journal transactions. See above.
+-}
+data CsvRules = CsvRules {
+      dateField :: Maybe FieldPosition,
+      dateFormat :: Maybe String,
+      statusField :: Maybe FieldPosition,
+      codeField :: Maybe FieldPosition,
+      descriptionField :: [FormatString],
+      amountField :: Maybe FieldPosition,
+      amountInField :: Maybe FieldPosition,
+      amountOutField :: Maybe FieldPosition,
+      currencyField :: Maybe FieldPosition,
+      baseCurrency :: Maybe String,
+      accountField :: Maybe FieldPosition,
+      account2Field :: Maybe FieldPosition,
+      effectiveDateField :: Maybe FieldPosition,
+      baseAccount :: AccountName,
+      accountRules :: [AccountRule]
+} deriving (Show, Eq)
+
+type FieldPosition = Int
+
+type AccountRule = (
+   [(String, Maybe String)] -- list of regex match patterns with optional replacements
+  ,AccountName              -- account name to use for a transaction matching this rule
+  )
+
+-- format strings
+
+data HledgerFormatField =
+    AccountField
+  | DefaultDateField
+  | DescriptionField
+  | TotalField
+  | DepthSpacerField
+  | FieldNo Int
+    deriving (Show, Eq)
+
+data FormatString =
+    FormatLiteral String
+  | FormatField Bool        -- Left justified ?
+                (Maybe Int) -- Min width
+                (Maybe Int) -- Max width
+                HledgerFormatField       -- Field
+  deriving (Show, Eq)
+
 
 data Ledger = Ledger {
-      journal :: Journal,
-      accountnametree :: Tree AccountName,
-      accountmap :: Map.Map AccountName Account
+      ledgerJournal :: Journal,
+      ledgerAccountNameTree :: Tree AccountName,
+      ledgerAccountMap :: Map.Map AccountName Account
     }
 
 data Account = Account {
       aname :: AccountName,
       apostings :: [Posting],    -- ^ postings in this account
       abalance :: MixedAmount    -- ^ sum of postings in this account and subaccounts
-    }
-
--- | A generic, pure specification of how to filter (or search) transactions and postings.
-data FilterSpec = FilterSpec {
-     datespan  :: DateSpan   -- ^ only include if in this date span
-    ,cleared   :: Maybe Bool -- ^ only include if cleared\/uncleared\/don't care
-    ,real      :: Bool       -- ^ only include if real\/don't care
-    ,empty     :: Bool       -- ^ include if empty (ie amount is zero)
-    ,acctpats  :: [String]   -- ^ only include if matching these account patterns
-    ,descpats  :: [String]   -- ^ only include if matching these description patterns
-    ,depth     :: Maybe Int
-    } deriving (Show)
+    } -- deriving (Eq)  XXX
 
