@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-|
 
 Standard imports and utilities which are useful everywhere, or needed low
@@ -19,12 +20,15 @@ module Hledger.Utils (---- provide these frequently used modules - or not, for c
                           -- module Text.Printf,
                           ---- all of this one:
                           module Hledger.Utils,
-                          Debug.Trace.trace
-                          ---- and this for i18n - needs to be done in each module I think:
-                          -- module Hledger.Utils.UTF8
+                          Debug.Trace.trace,
+                          -- module Hledger.Utils.UTF8IOCompat
+                          SystemString,fromSystemString,toSystemString,error',userError'
+                          -- the rest need to be done in each module I think
                           )
 where
-import Codec.Binary.UTF8.String as UTF8 (decodeString, encodeString, isUTF8Encoded)
+import Control.Monad (liftM)
+import Control.Monad.Error (MonadIO)
+import Control.Monad.IO.Class (liftIO)
 import Data.Char
 import Data.List
 import Data.Maybe
@@ -32,15 +36,17 @@ import Data.Time.Clock
 import Data.Time.LocalTime
 import Data.Tree
 import Debug.Trace
-import System.Info (os)
+import System.Directory (getHomeDirectory)
+import System.FilePath((</>), isRelative)
 import Test.HUnit
 import Text.ParserCombinators.Parsec
 import Text.Printf
 import Text.RegexPR
 -- import qualified Data.Map as Map
 -- 
--- import Prelude hiding (readFile,writeFile,getContents,putStr,putStrLn)
--- import Hledger.Utils.UTF8
+-- import Prelude hiding (readFile,writeFile,appendFile,getContents,putStr,putStrLn)
+-- import Hledger.Utils.UTF8IOCompat   (readFile,writeFile,appendFile,getContents,putStr,putStrLn)
+import Hledger.Utils.UTF8IOCompat (SystemString,fromSystemString,toSystemString,error',userError')
 
 -- strings
 
@@ -70,7 +76,12 @@ quoteIfSpaced :: String -> String
 quoteIfSpaced s | isSingleQuoted s || isDoubleQuoted s = s
                 | not $ any (`elem` s) whitespacechars = s
                 | otherwise = "'"++escapeSingleQuotes s++"'"
-                  where escapeSingleQuotes = regexReplace "'" "\'"
+
+escapeSingleQuotes :: String -> String
+escapeSingleQuotes = regexReplace "'" "\'"
+
+escapeQuotes :: String -> String
+escapeQuotes = regexReplace "([\"'])" "\\1"
 
 -- | Quote-aware version of words - don't split on spaces which are inside quotes.
 -- NB correctly handles "a'b" but not "''a''".
@@ -87,6 +98,7 @@ words' = map stripquotes . fromparse . parsewith p
 unwords' :: [String] -> String
 unwords' = unwords . map singleQuoteIfNeeded
 
+-- | Single-quote this string if it contains whitespace or double-quotes
 singleQuoteIfNeeded s | any (`elem` s) whitespacechars = "'"++s++"'"
                       | otherwise = s
 
@@ -125,8 +137,8 @@ concatBottomPadded strs = intercalate "\n" $ map concat $ transpose padded
       lss = map lines strs
       h = maximum $ map length lss
       ypad ls = ls ++ replicate (difforzero h (length ls)) ""
-      xpad ls = map (padleft w) ls where w | null ls = 0
-                                           | otherwise = maximum $ map length ls
+      xpad ls = map (padright w) ls where w | null ls = 0
+                                            | otherwise = maximum $ map length ls
       padded = map (xpad . ypad) lss
 
 -- | Compose strings vertically and right-aligned.
@@ -180,41 +192,6 @@ fitto w h s = intercalate "\n" $ take h $ rows ++ repeat blankline
       fit w = take w . (++ repeat ' ')
       blankline = replicate w ' '
 
--- encoded platform strings
-
--- | A platform string is a string value from or for the operating system,
--- such as a file path or command-line argument (or environment variable's
--- name or value ?). On some platforms (such as unix) these are not real
--- unicode strings but have some encoding such as UTF-8. This alias does
--- no type enforcement but aids code clarity.
-type PlatformString = String
-
--- | Convert a possibly encoded platform string to a real unicode string.
--- We decode the UTF-8 encoding recommended for unix systems
--- (cf http://www.dwheeler.com/essays/fixing-unix-linux-filenames.html)
--- and leave anything else unchanged.
-fromPlatformString :: PlatformString -> String
-fromPlatformString s = if UTF8.isUTF8Encoded s then UTF8.decodeString s else s
-
--- | Convert a unicode string to a possibly encoded platform string.
--- On unix we encode with the recommended UTF-8
--- (cf http://www.dwheeler.com/essays/fixing-unix-linux-filenames.html)
--- and elsewhere we leave it unchanged.
-toPlatformString :: String -> PlatformString
-toPlatformString = case os of
-                     "unix" -> UTF8.encodeString
-                     "linux" -> UTF8.encodeString
-                     "darwin" -> UTF8.encodeString
-                     _ -> id
-
--- | A version of error that's better at displaying unicode.
-error' :: String -> a
-error' = error . toPlatformString
-
--- | A version of userError that's better at displaying unicode.
-userError' :: String -> IOError
-userError' = userError . toPlatformString
-
 -- math
 
 difforzero :: (Num a, Ord a) => a -> a -> a
@@ -378,21 +355,21 @@ getCurrentLocalTime = do
 -- testing
 
 -- | Get a Test's label, or the empty string.
-tname :: Test -> String
-tname (TestLabel n _) = n
-tname _ = ""
+testName :: Test -> String
+testName (TestLabel n _) = n
+testName _ = ""
 
 -- | Flatten a Test containing TestLists into a list of single tests.
-tflatten :: Test -> [Test]
-tflatten (TestLabel _ t@(TestList _)) = tflatten t
-tflatten (TestList ts) = concatMap tflatten ts
-tflatten t = [t]
+flattenTests :: Test -> [Test]
+flattenTests (TestLabel _ t@(TestList _)) = flattenTests t
+flattenTests (TestList ts) = concatMap flattenTests ts
+flattenTests t = [t]
 
 -- | Filter TestLists in a Test, recursively, preserving the structure.
-tfilter :: (Test -> Bool) -> Test -> Test
-tfilter p (TestLabel l ts) = TestLabel l (tfilter p ts)
-tfilter p (TestList ts) = TestList $ filter (any p . tflatten) $ map (tfilter p) ts
-tfilter _ t = t
+filterTests :: (Test -> Bool) -> Test -> Test
+filterTests p (TestLabel l ts) = TestLabel l (filterTests p ts)
+filterTests p (TestList ts) = TestList $ filter (any p . flattenTests) $ map (filterTests p) ts
+filterTests _ t = t
 
 -- | Simple way to assert something is some expected value, with no label.
 is :: (Eq a, Show a) => a -> a -> Assertion
@@ -425,3 +402,19 @@ isRight = not . isLeft
 -- | Apply a function the specified number of times. Possibly uses O(n) stack ?
 applyN :: Int -> (a -> a) -> a -> a
 applyN n f = (!! n) . iterate f
+
+-- | Convert a possibly relative, possibly tilde-containing file path to an absolute one,
+-- given the current directory. ~username is not supported. Leave "-" unchanged. 
+expandPath :: MonadIO m => FilePath -> FilePath -> m FilePath -- general type sig for use in reader parsers
+expandPath _ "-" = return "-"
+expandPath curdir p = (if isRelative p then (curdir </>) else id) `liftM` expandPath' p
+  where
+    expandPath' ('~':'/':p)  = liftIO $ (</> p) `fmap` getHomeDirectory
+    expandPath' ('~':'\\':p) = liftIO $ (</> p) `fmap` getHomeDirectory
+    expandPath' ('~':_)      = error' "~USERNAME in paths is not supported"
+    expandPath' p            = return p
+
+firstJust ms = case dropWhile (==Nothing) ms of
+    [] -> Nothing
+    (md:_) -> md
+
